
package Text::Editor::Vip::View::Plugins::IncrementalFind ;

use strict;
use warnings ;

BEGIN 
{
use Exporter ();

use vars qw ($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
$VERSION     = 0.01;
@ISA         = qw (Exporter);
@EXPORT      = qw ();
@EXPORT_OK   = qw ();
%EXPORT_TAGS = ();
}

die "Not ported to Vip yet!" ;

#-------------------------------------------------------------------------------

1 ;

=head1 NAME

Text::Editor::Vip::View::Plugins::IncrementalFind- Incremental find functionality plugin for Vip::View

=head1 SYNOPSIS

=head1 DESCRIPTION

IncrementalFind- Incremental find functionality plugin for Vip::View

=head1 AUTHOR

	Khemir Nadim ibn Hamouda
	CPAN ID: NKH
	mailto:nadim@khemir.net
	http:// no web site

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=cut
